package es.prodevelop.cit.gvsig.arcims.gui.dialogs;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.raster.RasterFilterStackManager;
import org.cresques.px.Extent;
import org.cresques.px.PxRaster;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.EnhancedPanel;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.RasterTransparencyPanel;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.View;
import com.iver.andami.ui.mdiManager.ViewInfo;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;

import es.prodevelop.cit.gvsig.arcims.fmap.layers.FRasterLyrArcIMS;

import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.StatusLayerRaster;

/**
 * <P>
 * Dialog for the raster properties of a ArcIMS layer.
 * It manages the avants and aplies
 * filters to the raster through the filter stack manager according to the user's
 * selection. This dialog contains some panels.
 * </P>
 * <UL>
 * <LI>Propierties</LI>
 * <LI>Band selection</LI>
 * <LI>Transparency</LI>
 * <LI>Enhancement</LI>
 * </UL>
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class ArcImsRasterPropsDialog extends FilterRasterDialogPanel implements View {
	
	private IProjection 							currentProjection = null;
	private FRasterLyrArcIMS								fLayer = null;
	private static final int						nprops = 11;
	private Object[][]								props = null;
	private RasterFilterStackManager 				stackManager = null;
	private Status									status = null;
	private StatusLayerRaster						rasterStatus = null;
	private JFileChooser							fileChooser = null;
	private String									lastPath = new String("./");
	private	PxRaster								px = null;
	private GeoRasterFile							grf = null;
	private static final long serialVersionUID = 0;
	
	/**
	 * Class that holds the dialog state and restores the initial state if it
	 * is cancelled.
	 * @author Nacho Brodin <brodin_ign@gva.es>
	 */
	class Status{
		public String					inicAlpha;
		public int 						bandR;
		public int 						bandG;
		public int 						bandB;
		private ArrayList				filters = new ArrayList();
		
		public Status(String alpha, int bandR, int bandG, int bandB){
			this.inicAlpha = alpha;
			this.bandR = bandR;
			this.bandG = bandG;
			this.bandB = bandB;
			filters = stackManager.getStringsFromStack();
		}
				
		/**
		 * Restaura el Estado salvado 
		 */
		public void restoreStatus(ArcImsRasterPropsDialog props){
			// Reset the filter stack to the original state
			// if(stackManager != null) stackManager.deleteTempFilters();
			// Reset alpha

			int opac = Integer.parseInt(status.inicAlpha);
			opac = (int)((opac*255)/100);
			fLayer.setTransparency(255-opac);
			
			if (fLayer != null) {
				fLayer.setBandR(bandR);
				fLayer.setBandG(bandG);
				fLayer.setBandB(bandB);
			}
			
			// Restore filters
			if(filters!=null)
				stackManager.createStackFromStrings(filters);
			
			fLayer.getFMap().invalidate();
		}
	}
	
	public class DriverFileFilter extends FileFilter{
		
		private Driver driver;
		
		public DriverFileFilter(String driverName) throws DriverLoadException{
			driver = LayerFactory.getDM().getDriver(driverName);
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
		 */
		public boolean accept(File f) {
			if (f.isDirectory()) return true;
			if (driver instanceof RasterDriver){
				return ((RasterDriver) driver).fileAccepted(f);
			}else{
				throw new RuntimeException("Tipo no reconocido");
			}
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#getDescription()
		 */
		public String getDescription() {
			return ((Driver) driver).getName();
		}
	}
	

	/**
	 * Dialog window constructor.
	 * 
	 * @param layer the layer
	 * @param rangeR r band
	 * @param rangeG g band
	 * @param rangeB b band
	 */
	public ArcImsRasterPropsDialog(FRasterLyrArcIMS layer, int[][] rangeR, int[][] rangeG, int[][] rangeB){
		super();
		fLayer = layer;
		this.px = layer.getPxRaster();
		this.grf = layer.getGeoRasterFile();
		if(fLayer.getStatus()==null){
			rasterStatus = new StatusLayerRaster();
			fLayer.setStatus(rasterStatus);
		}else
			rasterStatus = (StatusLayerRaster)fLayer.getStatus();
		initialize();
		this.setRanges(rangeR, rangeG, rangeB);
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		fr.getBandSetup().getFileList().getJButtonAdd().setEnabled(false);
		fr.getBandSetup().getFileList().getJButtonRemove().setEnabled(false);
		this.setTranslation();
		
	}
	
	/**
	 * Sets the panels' texts
	 */
	private void setTranslation(){
		this.getBandSetup().getFileList().getJButtonAdd().setText(PluginServices.getText(this,"add"));
		this.getBandSetup().getFileList().getJButtonRemove().setText(PluginServices.getText(this,"remove"));
		this.getBandSetup().getFileList().lbandasVisibles.setText(PluginServices.getText(this,"bands"));
		
		RasterTransparencyPanel tpan = this.getTransparencyPanel();
		
		tpan.lGreenValue.setText(PluginServices.getText(this,"Valor_verde"));
		tpan.lRedValue.setText(PluginServices.getText(this,"Valor_rojo"));
		tpan.lBlueValue.setText(PluginServices.getText(this,"Valor_azul"));
		tpan.getTransparencyCheck().setText(PluginServices.getText(this,"transparencia"));
		tpan.getOpacityCheck().setText(PluginServices.getText(this,"opacidad"));
		tpan.lRange.setText(PluginServices.getText(this,"usar_rango"));
		tpan.lPixelValue.setText(PluginServices.getText(this,"valor_pixel")+": 0 a 255");
		
		EnhancedPanel ep = this.getEnhancedPanel();
		ep.lLineal.setText(PluginServices.getText(this,"lineal_directo"));
		ep.lQueue.setText(PluginServices.getText(this,"recorte_colas"));
		ep.lWithoutEnhanced.setText(PluginServices.getText(this,"sin_realce"));
		ep.lCut.setText(PluginServices.getText(this,"recorte"));
		ep.lRemove.setText(PluginServices.getText(this,"eliminar_extremos"));
		
		// Text translation
		for(int i=0;i<this.getTab().getTabCount();i++){
			if(this.getTab().getTitleAt(i).equals("Info"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"info"));
			if(this.getTab().getTitleAt(i).equals("Transparencia"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"transparencia"));
			if(this.getTab().getTitleAt(i).equals("Bands"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"bands"));
			if(this.getTab().getTitleAt(i).equals("Realce"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"realce"));
		}
		
		this.getAcceptButton().setText(PluginServices.getText(this,"ok"));
		this.getApplyButton().setText(PluginServices.getText(this,"apply"));
		this.getCancelButton().setText(PluginServices.getText(this,"cancel"));
	}
	
	/**
	 * Assigns a FLayerRaster
	 * @param layer	layer to be assigned
	 */
	public void setFLyrWMS(FRasterLyrArcIMS layer){
		fLayer = layer;
	}
	
	/**
	 * Dialog window constructor.
	 */
	public ArcImsRasterPropsDialog() {
		initialize();
	}
			
	/**
	 * Loads the info panel data.
	 */
	private void loadInfoData(){

		if(fLayer!=null){
			props = new Object[nprops][2];
			props[0][0] = new String(PluginServices.getText(this,"Fichero"));
			props[0][1] = "http://" + fLayer.getHost().getHost() + fLayer.getHost().getFile();
			props[1][0] = new String(PluginServices.getText(this,"num_bandas"));
			props[1][1] = new String(String.valueOf( grf.getBandCount()));
			props[2][0] = new String(PluginServices.getText(this,"ancho_alto"));
			props[2][1] = (int)(fLayer.getWidth())+" X "+(int)(fLayer.getHeight());
			props[3][0] = new String(PluginServices.getText(this,"format"));
			props[3][1] = fLayer.getFormat();
			props[4][0] = new String(PluginServices.getText(this,"tipo_dato"));
			String type = null;
			switch(grf.getDataType()){
				case 0: type = new String("BYTE");break;
				case 1: type = new String("USHORT");break;
				case 2: type = new String("SHORT");break;
				case 3: type = new String("INT");break;
				case 4: type = new String("FLOAT");break;
				case 5: type = new String("DOUBLE");break;
				default: type = new String("UNDEFINED");break;
			}
		    props[4][1] = type;
			props[5][0] = new String(PluginServices.getText(this,"coor_geograficas"));
			props[6][0] = new String(PluginServices.getText(this,"xmin"));
			props[6][1] = String.valueOf(px.getExtent().minX());
			props[7][0] = new String(PluginServices.getText(this,"ymin"));
			props[7][1] = String.valueOf(px.getExtent().minY());
			props[8][0] = new String(PluginServices.getText(this,"xmax"));
			props[8][1] = String.valueOf(px.getExtent().maxX());
			props[9][0] = new String(PluginServices.getText(this,"ymax"));
			props[9][1] = String.valueOf(px.getExtent().maxY());
			
		}else{
			props = new Object[1][2];
			props[0][0] = new String("No props");
			props[0][1] = new String("-");
		}
		
	}
	
	/**
	 * Adds bands to the FilterRasterDialogPanel's  band coounter 
	 * @param numBands Nmero de bandas a aadir
	 */
	public void addNumBands(int numBands){
		nbands += numBands;
		if(this.getTransparencyPanel() != null && this.getTransparencyPanel().getTRojo().isEnabled())
			this.getTransparencyPanel().setActiveTransparencyControl(true); 
	}
	
	/**
	 * Inits the jDialog	
	 */    
	private void initialize() {
		
		setName("filterRaster");
			   		
   		this.loadInfoData();
		super.init(props);
		this.setTabVisible("Pansharpening", false);

		this.setSize(486, 318);

		this.getAcceptButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				acceptButtonActionPerformed(evt);
				closeJDialog();
			}
		});
		this.getCancelButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
				closeJDialog();
			}
		});
		this.getApplyButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				acceptButtonActionPerformed(evt);
			}
		});
		this.getBandSetup().getFileList().getJButtonAdd().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt){
				addFileBand(evt);
				
			}
		});
		this.getBandSetup().getFileList().getJButtonRemove().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt){
				delFileBand(evt);
			}
		});

	
	}
	
	/**
	 * Saves the initial state to restore it if cancel.
	 */
	public void readStat(){
		status = new Status(((FilterRasterDialogPanel)this).getTransparencyPanel().getOpacityText().getText(),
							getAssignedBand(GeoRasterFile.RED_BAND),
							getAssignedBand(GeoRasterFile.GREEN_BAND),
							getAssignedBand(GeoRasterFile.BLUE_BAND)
							);				
	}
	
	/**
	 * This method initializes jContentPane		
	 */    
	public Container getContentPane() {
		return this;
	}
	
	/**
	 * Sets a projection.
	 * 
	 * @param prj
	 */
	public void setProjection(IProjection prj) {
		this.currentProjection = prj;
	}
	
	/**
	 * Gets the projection.
	 * 
	 * @return the projection
	 */
	public IProjection getProjection() {
		return this.currentProjection;
	}
	
	
	
	public void closeJDialog() {
		PluginServices.getMDIManager().closeView(ArcImsRasterPropsDialog.this);
	}
	
	/**
	 * Sets the RasterFilterStackManager
	 * @param stackManager
	 */
	public void setRasterFilterStackManager(RasterFilterStackManager stackManager){
		this.stackManager = stackManager;
		stackManager.resetTempFilters();
	}
	
	/**
	 * 
	 * @param flag the flag to be assigned
	 * 
	 * @return the assigned band
	 */
	public int getAssignedBand(int flag) {
		return this.getBandSetup().getAssignedBand(flag);
	}
	
	/**
	 * Pressing OK with the bands panel selected makes its values to be
	 * processed.
	 * 
	 * @return true if the bands panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processBandPanel(){
		fLayer.setBandR(getAssignedBand(GeoRasterFile.RED_BAND));
		fLayer.setBandG(getAssignedBand(GeoRasterFile.GREEN_BAND));
		fLayer.setBandB(getAssignedBand(GeoRasterFile.BLUE_BAND));
		rasterStatus.bandR = getAssignedBand(GeoRasterFile.RED_BAND);
		rasterStatus.bandG = getAssignedBand(GeoRasterFile.GREEN_BAND);
		rasterStatus.bandB = getAssignedBand(GeoRasterFile.BLUE_BAND);
			
		// Check whether there is any non-assigned band
		// and apply the filter
		StringBuffer sb = new StringBuffer();
		if(getAssignedBand(GeoRasterFile.RED_BAND) == -1)
			sb.append("R");
		if(getAssignedBand(GeoRasterFile.GREEN_BAND) == -1)
			sb.append("G");
		if(getAssignedBand(GeoRasterFile.BLUE_BAND) == -1)
			sb.append("B");
			
		if(!sb.toString().equals(""))
			stackManager.addRemoveBands(sb.toString());
		else
			stackManager.removeFilter(stackManager.getTypeFilter("removebands"));
			
		fLayer.getFMap().invalidate();
		return true;
	}
	
	/**
	 * Pressing OK with the transparency panel selected makes its values to be processed.
	 * @return true if the transparency panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processTransparencyPanel(){
		// opacity
		String sOpac = this.getTransparencyPanel().getOpacityText().getText();
		if(!sOpac.equals("") && this.getTransparencyPanel().getOpacityCheck().isSelected()){
			int opac = Integer.parseInt(sOpac);
			opac = (int)((opac*255)/100);
			px.setTransparency(true);
			//px.setTransparency(255-opac);
			fLayer.setTransparency(255-opac);
			rasterStatus.transparency = 255-opac;
		}else{
			px.setTransparency(false);
			fLayer.setTransparency(0);
			rasterStatus.transparency = 0;
		}
				
		// Transparency
		if(	this.getTransparencyPanel().getTransparencyCheck().isSelected()){
			this.checkTransparencyValues();
			stackManager.addTransparencyFilter(	this.getRangeRed(),
													this.getRangeGreen(),
													this.getRangeBlue(),
													0x10,	//Transparencia
													0xff,	//Color Transparencia R
													0xff,	//Color Transparencia G
													0xff);	//Color Transparencia B
		}else
			px.filterStack.removeFilter(stackManager.getTypeFilter("transparency"));
			
		fLayer.getFMap().invalidate();
		return true;	
	}
	
	/**
	 * Pressing OK with the enhancement panel selected makes its values to be processed.
	 * @return true if the enhancement panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processEnhancedPanel(){
		// Selected filter
		if(	this.getEnhancedPanel().getLinealDirectoRadioButton().isSelected()){
			if(	this.getEnhancedPanel().getRemoveCheck().isSelected() &&
				!this.getEnhancedPanel().getTailCheck().isSelected())
				stackManager.addEnhancedFilter(true);
			else
				stackManager.addEnhancedFilter(false);
							
			// Enhancement selected
			if(this.getEnhancedPanel().getTailCheck().isSelected()){
				stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
				double recorte = Double.parseDouble(this.getEnhancedPanel().getTailText().getText())/100;
				if(this.getEnhancedPanel().getRemoveCheck().isSelected())
					stackManager.addTailFilter( recorte, 0D, true);
				else
					stackManager.addTailFilter( recorte, 0D, false);								
			}else{
				stackManager.removeFilter(stackManager.getTypeFilter("tail"));
				stackManager.addComputeMinMaxFilter();
			}	
		}
			
		// No filter selected
		if(this.getEnhancedPanel().getSinRealceRadioButton().isSelected()){
			stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
			stackManager.removeFilter(stackManager.getTypeFilter("tail"));
			stackManager.removeFilter(stackManager.getTypeFilter("enhanced"));
		}
		fLayer.getFMap().invalidate();
			
		return true;
	}
	
	/**
	 * Manages the action perfomed when Apply/OK is pressed or Apply at the raster
	 * properties control.
	 * @param e
	 */
	private void acceptButtonActionPerformed(ActionEvent e) {
		this.processBandPanel();
		this.processTransparencyPanel();	
		this.processEnhancedPanel();	
	}
	
	/**
	 * Adds a band to the raster.
	 * @param e
	 */
	private void addFileBand(ActionEvent e){
		String[] driverNames = null;
		
		fileChooser = new JFileChooser(lastPath);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setAcceptAllFileFilterUsed(false);
        try {
			driverNames = LayerFactory.getDM().getDriverNames();
			FileFilter auxF;
			for (int i = 0; i < driverNames.length; i++) {
				
				if (driverNames[i].endsWith("gvSIG Image Driver")){
				    auxF = new DriverFileFilter(driverNames[i]);
					fileChooser.addChoosableFileFilter(auxF);
				}
			}
		} catch (DriverLoadException e1) {
			NotificationManager.addError("No se pudo acceder a los drivers", e1);
		}
		int result = fileChooser.showOpenDialog(ArcImsRasterPropsDialog.this);
		
		if(result == JFileChooser.APPROVE_OPTION){
			File[] files = fileChooser.getSelectedFiles();
		 	BandSetupPanel bandSetup = ((FilterRasterDialogPanel)this.getContentPane()).getBandSetup();
		 	lastPath = files[0].getPath();
		 	
		 	// Add it to the layer if not present
		 	Vector v = new Vector();
            for(int i=0;i<files.length;i++){
            	boolean exist = false;
            	for(int j=0;j<px.getFiles().length;j++){
            		if(px.getFiles()[j].getName().endsWith(files[i].getName()))
            			exist = true;
            	}
            	if(!exist){
            		try{
            			Rectangle2D extentOrigin = fLayer.getFullExtent();
            			
            			Extent extentNewFile = GeoRasterFile.openFile(fLayer.getProjection(), files[i].getAbsolutePath()).getExtent();
            			
						// Check that the extent and the added file size
            			// matches the original. If not, layer is not opened and a warning is
            			// shown.
            			double widthNewFile = (extentNewFile.getMax().getX()-extentNewFile.getMin().getX());
            			double heightNewFile = (extentNewFile.getMax().getY()-extentNewFile.getMin().getY());
            				                                                                                                
            			if( (widthNewFile-extentOrigin.getWidth()) > 1.0 ||
            				(widthNewFile-extentOrigin.getWidth()) < -1.0 ||
            				(heightNewFile-extentOrigin.getHeight()) > 1.0 ||
            				(heightNewFile-extentOrigin.getHeight()) < -1.0){       
            				JOptionPane.showMessageDialog(	null,
            												PluginServices.getText(this, "extents_no_coincidentes"), 
															"",
															JOptionPane.ERROR_MESSAGE);
            				return;
            			}
            				                                
            			if(	(extentNewFile.getMax().getX()-extentNewFile.getMin().getX())!=extentOrigin.getWidth() ||
								(extentNewFile.getMax().getY()-extentNewFile.getMin().getY())!=extentOrigin.getHeight()	){
            				JOptionPane.showMessageDialog(null, 
									PluginServices.getText(this, "extents_no_coincidentes"), "", JOptionPane.ERROR_MESSAGE);
							return;
            			}
														
            		}catch(Exception exc){
            			exc.printStackTrace();
            		}
            		
            		// Add it to the layer
            		px.addFile(files[i].getAbsolutePath());

            	}else{
            		JOptionPane.showMessageDialog(null, 
							PluginServices.getText(this, "fichero_existe")+" "+files[i].getAbsolutePath(), "", JOptionPane.ERROR_MESSAGE);
            	}
            }
		 	            
            // Add georasterfile to the panel's table
            v = new Vector();
            for(int i=0;i<px.getFiles().length;i++){
            	boolean exist = false;
            	for(int j=0;j<bandSetup.getNBands();j++){
            		if(px.getFiles()[i].getName().endsWith(bandSetup.getBandName(j)))
            			exist = true;
            	}
            	if(!exist)
            		v.add(px.getFiles()[i]);
            }
            
            GeoRasterFile[] grf = new GeoRasterFile[v.size()];
            for(int i=0;i<grf.length;i++){
            	grf[i] = (GeoRasterFile)v.get(i);
            }
            bandSetup.addFiles(grf);
		}
	}
	
	/**
	 * Deletes a band from the raster. If there is only a file or no band is
	 * selected then it does nothing.
	 * 
	 * @param e
	 */
	private void delFileBand(ActionEvent e){
		BandSetupPanel bandSetup = ((FilterRasterDialogPanel)this.getContentPane()).getBandSetup();
	
		if(	bandSetup.getFileList().getJList().getSelectedValue()!=null &&
			bandSetup.getFileList().getNFiles() > 1){
			String pathName = bandSetup.getFileList().getJList().getSelectedValue().toString();
			px.delFile(pathName);
			String file = pathName.substring(pathName.lastIndexOf("/")+1);
			file = file.substring(file.lastIndexOf("\\")+1);
			bandSetup.removeFile(file);
			
		}		
	}
	
	/**
	 * The cancel button restores the previous state to the loading of this dialog
	 * @param e	Evento
	 */
	private void cancelButtonActionPerformed(ActionEvent e) {
		this.status.restoreStatus(this);
		fLayer.getFMap().invalidate();
	}
	
	public ViewInfo getViewInfo() {
		ViewInfo m_viewinfo=new ViewInfo(ViewInfo.MODALDIALOG);
    		m_viewinfo.setTitle(PluginServices.getText(this, "propiedades_raster"));
		return m_viewinfo;
	}
	
	
}  
