/**
 * 
 */
package org.gvsig.remoteClient.arcims;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.gvsig.remoteClient.arcims.exceptions.ArcImsException;
import org.gvsig.remoteClient.arcims.utils.ArcImsDownloadUtils;
import org.gvsig.remoteClient.arcims.utils.GetFeaturesTags;
import org.gvsig.remoteClient.arcims.utils.GetVariables;
import org.gvsig.remoteClient.arcims.utils.ServiceInfoTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

import com.hardcode.gdbms.engine.values.IntValue;
import com.hardcode.gdbms.engine.values.ValueFactory;

/**
 * @author jsanz
 *
 */
public class IdsReader{

	private static Logger logger = Logger.getLogger(IdsReader.class.getName());
	
	ArcImsVectStatus status;
	URL url;
	boolean hasMore;
	int featCount;
	BufferedReader lector;
	String idName;
	KXmlParser parser;
	int tag;
	
	
	/**
	 * @param in
	 */
	public IdsReader(ArcImsVectStatus status, String idName) {
		try {
			this.status = status;
			this.idName = idName;
			url = new URL(buildCapabilitiesRequest(status));
			parser = new KXmlParser();
			hasMore = true;
			featCount = 0;
		} 
		catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	private void initReader() {
		this.parser = null;
		this.lector = null;
		
		try {
			/*
			 * Build the proper ArcXML
			 */
			String request = ArcXMLFeatures.getIdsRequest(status,featCount,idName);
			if (status.verbose) System.err.println(request);
			this.lector =  new BufferedReader(new InputStreamReader(ArcImsDownloadUtils.getRemoteIS(url,request)));
			this.parser.setInput(lector);
		} 
		catch (ArcImsException e) {
			e.printStackTrace();
			this.parser = null;
			this.lector = null;
		}
		catch (XmlPullParserException e) {
			e.printStackTrace();
			this.parser = null;
			this.lector = null;
		}
	}

	private String buildCapabilitiesRequest(ArcImsVectStatus mstatus) {
		String onlineResource = mstatus.getServer();
    	String service = mstatus.getService();
    	
		StringBuffer req = new StringBuffer();		
		req.append(onlineResource);
		req.append("?"+GetVariables.SERVICENAME+"=");
		req.append(service);
		return req.toString();
	}

	public IntValue readId() throws IOException, ArcImsException {
		
		if (lector==null || lector.readLine()==null)
			initReader();
		
//		//If lector is null we will init new Reader
//		if (lector.readLine()==null) {
//			 initReader();
//		}
		/*
		 * If hasMore flag is false we have reached 
		 * the end of the last XML and we will return a null
		 */
		else if (hasMore==false)
			return null;
		
		int i = parseId();
		IntValue iV = ValueFactory.createValue(i);
		
		return iV;
	}

	
	public int parseId() throws ArcImsException {
		/*
		 * Start parsing
		 */
		int val = -1;
		boolean end = false;

		try {
			parser.nextTag();

			tag = parser.nextTag();

			do {
				switch (tag) {
				case KXmlParser.START_TAG:
					if (parser.getName().equals(GetFeaturesTags.FIELD)) {
						val = Integer.parseInt(parser.getAttributeValue("",GetFeaturesTags.VALUE));
					} 
					else if (parser.getName().equals(GetFeaturesTags.FEATURECOUNT)) {
						String value = new String();
						// Get the feature count
						value = parser.getAttributeValue("",GetFeaturesTags.COUNT);
						Integer intFeatCount = new Integer(value);
						featCount = intFeatCount.intValue();

						// Get the hasmore boolean
						value = parser.getAttributeValue("",GetFeaturesTags.HASMORE);
						Boolean boHasMore = new Boolean(value);
						hasMore = boHasMore.booleanValue();

						initReader();

						val = readId().getValue();
						end = true;
					} else if (parser.getName().equals(ServiceInfoTags.tERROR)) {
						logger.error("Error parsing GET_FEATURES:\r\n"
								+ parser.nextText());
						throw new ArcImsException("arcims_server_error");
					}
					break;
				case KXmlParser.END_TAG:
					if (parser.getName().equals(GetFeaturesTags.FEATURE))
						end = true;
					break;
				case KXmlParser.TEXT:
					break;
				}
				if (!end)
					tag = parser.next();
			} while (!end);
		} catch (XmlPullParserException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return val;
	}
	
}
